/*
============================================================================
 Name		: SocketsEngine.h
 Copyright   : Symbian Academy, 2007
 Description : The sockets engine is able to connect a socket and manages
               the reader and writer active objects.
============================================================================
*/

#ifndef SOCKETSENGINE_H
#define SOCKETSENGINE_H

#include <e32base.h>	// For CActive, link against: euser.lib
#include <e32std.h>

// Edit 1: Include the required header files for the classes that will be 
//         needed for working with sockets: RSocketServ, RSocket and TInetAddr
//         (see the SDK help)

#include "SocketsWriter.h"
#include "SocketsReader.h"
#include "MSocketsNotifier.h"

class CSocketsEngine : public CActive
{
public:
	// Cancel and destroy
	~CSocketsEngine();

	// Two-phased constructor.
	static CSocketsEngine* NewL(MSocketsNotifier* aNotifier);

	// Two-phased constructor.
	static CSocketsEngine* NewLC(MSocketsNotifier* aNotifier);

public: // New functions
	// Edit 2: ConnectL() is the method that can be executed from the
	//         outside to initiate the connection process.
	//         Add a parameter to supply the target address, which
	//         should be of type TInetAddr&
	void ConnectL( );
	
	TBool IsConnected();
	
	void IssueSend(const TDesC8& aMessage);

private:
	// C++ constructor
	CSocketsEngine(MSocketsNotifier* aNotifier);
	
	// Second-phase constructor
	void ConstructL();
	
private: // From CActive
	// Handle completion
	void RunL();
	
	// How to cancel me
	void DoCancel();
	
private:
	enum TSocketsEngineState
	{
		EUninitialized,
		EInitialized,	// Initalized
		EConnecting,
		EConnected
	};

private:
	// Edit 3: Define an instance variable for the socket server
	//         (of type RSocketServ)

	// Edit 4: Define an instance variable for the socket itself
	//         (of type RSocket)
	
	TInt	iEngineState;			// State of the engine

	MSocketsNotifier* iNotifier;

	CSocketsWriter* iWriter;
	CSocketsReader* iReader;
};

#endif // SOCKETSENGINE_H
