/*
 ============================================================================
 Name		 : SocketsWriter.h
 Copyright   : Symbian Academy, 2007
 Description : Responsible for sending data through the socket to the server.
 ============================================================================
 */

#ifndef SOCKETSWRITER_H
#define SOCKETSWRITER_H

#include <e32base.h>	// For CActive, link against: euser.lib
#include <e32std.h>

#include <es_sock.h>
#include "MSocketsNotifier.h"

class CSocketsWriter : public CActive
	{
public:
	// Cancel and destroy
	~CSocketsWriter();

	// Two-phased constructor.
	static CSocketsWriter* NewL(RSocket* aSocket, MSocketsNotifier* aNotifier);

	// Two-phased constructor.
	static CSocketsWriter* NewLC(RSocket* aSocket, MSocketsNotifier* aNotifier);

public:
	// New functions
	// Function for making the initial request
	void IssueSend( const TDesC8& aData);

private:
	// C++ constructor
	CSocketsWriter(RSocket* aSocket, MSocketsNotifier* aNotifier);

	// Second-phase constructor
	void ConstructL();

private:
	// From CActive
	// Handle completion
	void RunL();

	// How to cancel me
	void DoCancel();
	
private:
	enum TSocketsWriterState
		{
		EInitialized, // Initalized
		ESending,
		EError // Error condition
		};

private:
	TInt iState; // State of the active object
	
	// Edit 19: Define an 8 bit (!) buffer (e.g. of RBuf-type) that
	//          will be used to save a local copy of messages
	

	RSocket* iSocket; // owned by AppUi
	MSocketsNotifier* iNotifier; // owned by AppUi

	};

#endif // SOCKETSWRITER_H

